#
# Anbox - The Android in a Box runtime environment
# Copyright 2018 Canonical Ltd.  All rights reserved.
#

import http.client
import subprocess
from subprocess import check_call

import netifaces
from charmhelpers.contrib.charmsupport import nrpe
from charmhelpers.core import host
from charmhelpers.core.hookenv import (log, unit_public_ip, status_set, config,
                                       relation_ids, application_version_set)
from charmhelpers.core.host import service_restart, service
from charms.layer.anbox import (install_or_upgrade_anbox_package,
                                remove_anbox_package,
                                close_old_and_open_new_ports,
                                snap_set)
from charms.reactive import when, when_not, hook, when_any
from charms.reactive.flags import is_flag_set, set_flag, clear_flag
from charms.layer.aws import AWS_METADATA_SERVICE_ADDRESS, is_aws_cloud

SERVICE = 'snap.ams-node-controller.ams-node-controller.service'

DEB_PACKAGE_NAME = 'anbox-cloud-ams-node-controller'

AWS_NIC_SUBNET_ENDPOINT = "/latest/meta-data/network/interfaces/macs/{}/vpc-ipv4-cidr-block"


@when('ua.ready')
@when_not('ams-node-controller.installed')
def install():
    install_or_upgrade_anbox_package(DEB_PACKAGE_NAME)
    set_charm_version()
    check_call(['snap', 'connect', 'ams-node-controller:lxd', 'lxd'])
    check_call(['snap', 'connect', 'ams-node-controller:network-control',
                ':network-control'])
    check_call(['snap', 'connect', 'ams-node-controller:firewall-control',
                ':firewall-control'])
    set_flag('ams-node-controller.installed')
    status_set('active', '')


@hook('upgrade-charm')
def upgrade_charm():
    """Sets the charm as active after upgrading the charm"""
    if not is_flag_set('ams-node-controller.installed'):
        return

    install_or_upgrade_anbox_package(DEB_PACKAGE_NAME)

    set_charm_version()
    configure()

    # If we have an NRPE relation, force run of that as well
    if relation_ids('nrpe-exteral-master'):
        clear_flag('nagios.configured')


def set_charm_version():
    try:
        info = subprocess.run(['snap', 'info', 'ams-node-controller'],
                              stdout=subprocess.PIPE)
    except subprocess.CalledProcessError as err:
        log('Failed to get snap info: {}'.format(err), level='ERROR')
    else:
        stdout = info.stdout.decode('utf8').splitlines()
        version = [line.split()[1] for line in stdout if 'installed' in line]
        if version:
            application_version_set(version[0])


@hook('stop')
def stop():
    """Remove the charm main snap when charm is removed"""
    if not is_flag_set('ams-node-controller.installed'):
        return

    remove_anbox_package(DEB_PACKAGE_NAME)


@when('ams-node-controller.installed', 'endpoint.lxd.available')
def endpoint_lxd_available(lxd):
    """
    The relation with the lxd application is set and its
    resources, like the unix socket, are usable. Start this service
    in case it is not running
    """
    if not host.service_running(SERVICE):
        host.service_start(SERVICE)


@when('ams-node-controller.installed', 'config.changed.public_interface')
def configure():
    """
    If public-interface config is set, its value replaces as public
    endpoint the unit public IP (the default one)
    """
    public_interface = config('public_interface')
    if not public_interface:
        # AWS has a specific network typology, we configure the node controller
        # to account for it
        if is_aws_cloud() or is_flag_set('is_aws_cloud'):
            set_flag('is_aws_cloud')
            set_aws_public_interface()
            set_aws_subnet()
        else:
            snap_set('public-address', unit_public_ip())
            snap_set('public-interface', '')
    else:
        snap_set('public-address', '')
        snap_set('public-interface', config('public_interface'))

    # As we now have the node controller configured we can enable it. By
    # default the snap is disabled as it's missing the right configuration
    service('enable', SERVICE)

    service_restart(SERVICE)
    status_set('active', '')


@when('config.changed.port')
def config_changed_port():
    close_old_and_open_new_ports('port')


def set_aws_public_interface():
    """
    Set as the public interface the first found starting with 'en'
    or 'eth' prefixes. AWS is not always setting the same name
    for such interface in its machines.
    This works as long as AWS uses those interface prefixes in all
    their machines when deploying juju services on it
    """
    public_interface = find_aws_public_interface(['en', 'eth'])
    if not public_interface:
        log('Could not detect a public interface for AWS')
        return

    snap_set('public-address', '')
    snap_set('public-interface', public_interface)


def set_aws_subnet():
    """
    AWS interfaces have the Availability Zone subnet, not the VPC subnet.
    This means that unexposed services are not reachable across AZs.
    We fetch the instance metadata to get the VPC subnet and use it instead.
    """
    # Find the mac address for the interface we're interested in
    preferred_interfaces_prefixes = ['en', 'eth']
    interface = ''
    for iface in netifaces.interfaces():
        for prefix in preferred_interfaces_prefixes:
            if iface.startswith(prefix):
                interface = iface
                break

    if interface == '':
        log('Failed to determine subnet for AWS: No interface')
        return

    nic_mac_address = netifaces.ifaddresses(interface)[netifaces.AF_LINK][0]['addr']

    conn = http.client.HTTPConnection(AWS_METADATA_SERVICE_ADDRESS, 80, timeout=10)
    conn.request("GET", AWS_NIC_SUBNET_ENDPOINT.format(nic_mac_address))
    interfaces_req = conn.getresponse()

    if interfaces_req.status != 200:
        log('Failed to determine VPC subnet for AWS: Could not list interfaces: {}'.format(
            interfaces_req.reason))
        return

    snap_set('subnet', interfaces_req.read().decode('utf-8'))


def find_aws_public_interface(prefixes):
    """
    Detects the first public interface matching any of the provided
    prefixes and returns its name
    """
    for iface in netifaces.interfaces():
        for prefix in prefixes:
            if iface.startswith(prefix):
                return iface
    return None


@when_any('config.changed.nagios_context',
          'config.changed.nagios_servicegroups')
def nagios_config_changed():
    if not is_flag_set('endpoint.nrpe-external-master.joined'):
        return
    update_nrpe_config()


@when('nrpe-external-master.available')
@when_not('nagios.configured')
def update_nrpe_config():
    hostname = nrpe.get_nagios_hostname()
    nrpe_setup = nrpe.NRPE(hostname=hostname)
    cmd = [
        '/usr/lib/nagios/plugins/check_procs',
        '-c', '1:', '-C', 'node-controller'
    ]
    nrpe_setup.add_check(
        shortname='ams-node-controller_proc',
        description='Check AMS Node Controller process',
        check_cmd=" ".join(cmd),
    )
    nrpe_setup.write()
    set_flag('nagios.configured')
