#!/usr/bin/python
#
# Anbox - The Android in a Box runtime environment
# Copyright 2018 Canonical Ltd.  All rights reserved.
#

import unittest
import mock

from charms.layer.ams_node_controller import (
    extract_range_start, extract_range_end, close_old_and_open_new_ports)


class TestOpenAndClosePorts(unittest.TestCase):

    def test_extract_range_start(self):
        self.assertEquals(
            extract_range_start('4000-5000'), 4000)

    def test_extract_range_end(self):
        self.assertEquals(
            extract_range_end('4000-5000'), 5000)

    @mock.patch('charms.layer.ams_node_controller.open_port')
    @mock.patch('charms.layer.ams_node_controller.close_port')
    def test_none_new_port(self, mock_close_port, mock_open_port):
        close_old_and_open_new_ports(None, None)
        mock_close_port.assert_not_called
        mock_open_port.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_port')
    @mock.patch('charms.layer.ams_node_controller.close_port')
    def test_empty_new_port(self, mock_close_port, mock_open_port):
        close_old_and_open_new_ports(None, '')
        mock_close_port.assert_not_called
        mock_open_port.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_port')
    def test_open_ports(self, mock_open_port):
        close_old_and_open_new_ports(None, '10050-10100')
        mock_open_port.assert_called_with('10050-10100')

    @mock.patch('charms.layer.ams_node_controller.open_port')
    @mock.patch('charms.layer.ams_node_controller.close_port')
    def test_close_single_open_single_port_using_ints(self,
                                                      mock_close_port,
                                                      mock_open_port):
        close_old_and_open_new_ports(10000, 11000)
        mock_close_port.assert_called_with(10000)
        mock_open_port.assert_called_with(11000)

    @mock.patch('charms.layer.ams_node_controller.open_port')
    @mock.patch('charms.layer.ams_node_controller.close_port')
    def test_close_single_open_single_port_using_str(self,
                                                     mock_close_port,
                                                     mock_open_port):
        close_old_and_open_new_ports('10000', '11000')
        mock_close_port.assert_called_with(10000)
        mock_open_port.assert_called_with(11000)

    @mock.patch('charms.layer.ams_node_controller.open_port')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_range_open_single_port(self, mock_close_ports,
                                          mock_open_port):
        close_old_and_open_new_ports('10000-10500', '11000')
        mock_close_ports.assert_called_with(10000, 10500)
        mock_open_port.assert_called_with(11000)

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_port')
    def test_open_single_close_range_port(self, mock_close_port,
                                          mock_open_ports):
        close_old_and_open_new_ports('10000', '11000-11500')
        mock_close_port.assert_called_with(10000)
        mock_open_ports.assert_called_with(11000, 11500)

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_range_open_range_port(self, mock_close_ports,
                                         mock_open_ports):
        close_old_and_open_new_ports('10000-10500', '11000-11500')
        mock_close_ports.assert_called_with(10000, 10500)
        mock_open_ports.assert_called_with(11000, 11500)

    # Mising ranges
    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_open_range_ports_including_old_in_new(self,
                                                         mock_close_ports,
                                                         mock_open_ports):
        close_old_and_open_new_ports('10500-10700', '10000-11000')
        self.assertEquals(mock_open_ports.call_count, 2)
        expected_args = [mock.call(10701, 11000), mock.call(10000, 10499)]
        self.assertEquals(mock_open_ports.call_args_list, expected_args)
        mock_close_ports.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_open_range_ports_including_new_in_old(self,
                                                         mock_close_ports,
                                                         mock_open_ports):
        close_old_and_open_new_ports('10000-11000', '10500-10700')
        self.assertEquals(mock_close_ports.call_count, 2)
        expected_args = [mock.call(10000, 10499), mock.call(10701, 11000)]
        self.assertEquals(mock_close_ports.call_args_list, expected_args)
        mock_open_ports.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_open_range_ports_old_on_left(self, mock_close_ports,
                                                mock_open_ports):
        close_old_and_open_new_ports('10000-11000', '10500-12700')
        mock_close_ports.assert_called_once_with(10000, 10499)
        mock_open_ports.assert_called_once_with(11001, 12700)

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_open_range_ports_old_on_right(self, mock_close_ports,
                                                 mock_open_ports):
        close_old_and_open_new_ports('10500-12700', '10000-11000')
        mock_close_ports.assert_called_once_with(11001, 12700)
        mock_open_ports.assert_called_once_with(10000, 10499)

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_range_ports_start_equals(self, mock_close_ports,
                                            mock_open_ports):
        close_old_and_open_new_ports('10500-12700', '10500-11000')
        mock_close_ports.assert_called_once_with(11001, 12700)
        mock_open_ports.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_close_range_ports_end_equals(self, mock_close_ports,
                                          mock_open_ports):
        close_old_and_open_new_ports('10000-12700', '10500-12700')
        mock_close_ports.assert_called_once_with(10000, 10499)
        mock_open_ports.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_open_range_ports_start_equals(self, mock_close_ports,
                                           mock_open_ports):
        close_old_and_open_new_ports('10500-11000', '10500-12700')
        mock_open_ports.assert_called_once_with(11001, 12700)
        mock_close_ports.assert_not_called

    @mock.patch('charms.layer.ams_node_controller.open_ports')
    @mock.patch('charms.layer.ams_node_controller.close_ports')
    def test_open_range_ports_end_equals(self, mock_close_ports,
                                         mock_open_ports):
        close_old_and_open_new_ports('10500-12700', '10000-12700')
        mock_open_ports.assert_called_once_with(10000, 10499)
        mock_close_ports.assert_not_called
